"""
This example represents a simple example of plugin usage without
expanding to functionality such as versioning. 

In this example we consider the Zoo object as being the central component
and all animals are then represented by plugins. This means you can always
add more animals to the zoo without having to alter key functionality within
the Zoo class.

Using this example code you can see how we can build some (albeit basic)
functionality of a zoo without hard coding in any animal types. In this way
we can extend the contents of zoo to include new animals without ever having
to touch the core zoo code. This is a fundamental principal of the plugin
architecture.

.. code-block:: python

    >>> import random
    >>> import factories.examples.zoo
    >>> 
    >>> # -- Create a zoo
    >>> zoo = factories.examples.zoo.Zoo()
    >>> 
    >>> # -- Add a random amount of animals to our zoo
    >>> for i in range(random.randrange(5, 20)):
    >>> zoo.add_animal(
    ...         random.choice(
    ...             zoo.appropriate_animals(),
    ...         ),
    ...     )
    >>> 
    >>> # -- Now print off the pens in the zoo
    >>> print(
    ...     'Our zoo has the following pens: %s' % (
    ...         '; '.join(
    ...             zoo.pens()
    ...         ),
    ...     )
    ... )
    >>> 
    >>> # -- Now lets visit each pen
    >>> for pen in zoo.pens():
    >>>     zoo.visit_pen(pen)
    >>> 
    >>> # -- Now we will cycle time, printing our animal count
    >>> day = 0
    >>> while zoo.animals():
    ...     print('Day %s' % day)
    ...     print('\tToday our zoo has %s animals' % len(zoo.animals()))
    ...     zoo.next_day()
    ...     day += 5
    >>> 
    >>> print('All our animals have died.')

In this example we can continually add new animals and the functionality - or
in this case the print statements - will increase without having to edit the
zoo itself.
You can give this a try simply by either dropping a new .py file into the
animals subfolder, or by adding a new animal class into one of the pre-existing
.py files.
"""
from .zoo import (
    Zoo,
    Animal,
)



"""
This contains the suite of tests run over the Factory object in order
to guarantee is robustness.

Many of the tests utilise the factories in the example subpackage. If you
plan to contribute fixes or additional functionality you should ensure that
all of the pre-existing tests pass as well as having test coverage for any
additional changes.
"""



import os
import factories


# ------------------------------------------------------------------------------
class DataReader(object):
    """
    This class represents a generic fruit reader capable of parsing
    fruit data in multiple formats seamlessly
    """

    # --------------------------------------------------------------------------
    def __init__(self):
        self.factory = factories.Factory(
            abstract=ReaderPlugin,
            versioning_identifier='version',
            paths=[
                os.path.join(
                    os.path.dirname(__file__),
                    'readers',
                )
            ]
        )

    # --------------------------------------------------------------------------
    def read(self, filepath):
        """
        Generic read method which will cycle over all the available plugins
        and utilise the one which is capable of operating on the given file

        :param filepath: Absolute path to data to be read
        :type filepath: str

        :return: Dictionary of data stored in the file
        :rtype: dict
        """
        for plugin in self.factory.plugins():

            if plugin.can_read(filepath):
                return plugin.contents(filepath)

        raise Exception(
            'No plugin able to read : %s' % filepath
        )


# ------------------------------------------------------------------------------
class ReaderPlugin(object):

    version = 1

    def contents(self, filepath):
        """
        This should return a dictionary of data collated from the
        file.

        Note: This MUST be re-implemented in the plugin

        :param filepath: Absolute filepath to file
        :type filepath: str

        :return: dictionary of data taken from the file
        :rtype: dict
        """
        return dict()

    @classmethod
    def can_read(cls, filepath):
        """
        Checks whether this plugin is capable of operating
        on the given file

        :param filepath: Absolute path to the file to test
        :type filepath: str

        :return: True if the plugin can operate on the file
        :rtype: bool
        """
        return False



"""
This example shows how we can utilise a plugin/factory pattern to 
make it easy to read file contents easily without having to hard code
readers into the core of our system.

Instead our core contains a factory, and we add plugins which are capable
of handling different file types. We can then ask each plugin whether the
plugin is able to parse the file - the first plugin to say yes is used and the
resulting data is returned.

You can try this example by running:

.. code-block:: python

    >>> from factories.examples import reader
    >>> 
    >>> # -- Instance a reader
    >>> data_reader = reader.DataReader()
    >>> 
    >>> # -- Read some data from an ini file
    >>> data = data_reader.read(reader.ini_path)
    >>> print(data)
    >>> 
    >>> # -- Now read some json data
    >>> data = data_reader.read(reader.json_path)
    >>> print(data)

"""
import os

from .core import(
    DataReader,
    ReaderPlugin,
)

# -- Some convience variables for testing this example
ini_path = os.path.join(
    os.path.dirname(__file__),
    'data',
    'data.ini',
)

json_path = os.path.join(
    os.path.dirname(__file__),
    'data',
    'data.json',
)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='factories',
    version='1.2.0',
    author='Mike Malinowski',
    author_email='mike@twisted.space',
    description='A python package exposing the factory/plugin design pattern',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mikemalinowski/factories',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
